class CommandNotFound(KeyError):
    pass


class Command(object):

    template = None

    def __init__(self, command_string):
        if command_string.startswith('/'):
            command_string = command_string.lstrip('/')
        self.command_string = command_string
        self.parts = command_string.split()

    def to_irc(self):
        raise NotImplemented('You need to define a `to_irc` method')


class PrivmsgCommand(Command):

    template = 'PRIVMSG {target} :{message}'

    def to_irc(self):
        message = ' '.join(self.parts[2:])
        return self.template.format(
            target=self.parts[1], message=message)


class NickCommand(Command):

    template = 'NICK {nick}'

    def to_irc(self):
        return self.template.format(nick=self.parts[1])


COMMAND_MAP= {
    'msg': PrivmsgCommand,
    'nick': NickCommand,
}


def command_parser(command_str):
    if command_str.startswith('/'):
        command_parts = command_str.lstrip('/').split()
        try:
            command_cls = COMMAND_MAP[command_parts[0].lower()]
        except KeyError:
            raise CommandNotFound('/{command}... not found'.format(
                command=command_parts[0]))
        command = command_cls(command_str)
        return command.to_irc()
    else:
        return command_str



import pkg_resources

from clint.textui import colored
import redis

from tenyks.config import settings

version = pkg_resources.require('tenyks')[0].version

BANNER = [
    "  _                   _         ",
    " | |                 | |        ",
    " | |_ ___ _ __  _   _| | _____  ",
    " | __/ _ \ '_ \| | | | |/ / __| ",
    " | ||  __/ | | | |_| |   <\__ \ ",
    "  \__\___|_| |_|\__, |_|\_\___/ ",
    "                 __/ |          ",
    "                |___/           ",
]

version_string = "Version: {version}\n\n"


NETWORKS = """\
[Configuration]
\tDebug: {debug}
\tRedis: redis://{redis_host}:{redis_port} [{redis_status}]

[Networks]
{networks}
"""


def get_redis_status():
    rds = redis.Redis(**settings.REDIS_CONNECTION)
    try:
        rds.ping()
        status = colored.green('OK')
    except redis.ConnectionError:
        status = colored.red('FAILED')
    return status


def startup_banner():
    banner = '\n'.join(BANNER) + '\n\n'
    banner = banner + version_string + NETWORKS
    networks = []
    for name, connection in settings.CONNECTIONS.items():
        if 'ssl' in connection and connection['ssl']:
            name = name + ' (SSL)'
        networks.append('\t' + name)
    banner = banner.format(
        version=colored.green(version), networks='\n'.join(networks),
        debug=colored.green(
            str((settings.DEBUG if hasattr(settings, 'DEBUG') else False))),
        redis_host=settings.REDIS_CONNECTION['host'],
        redis_port=settings.REDIS_CONNECTION['port'],
        redis_status=get_redis_status())
    return banner



import os
from os.path import abspath, join, dirname
import sys
import logging.config
from logging.handlers import SysLogHandler

from jinja2 import Template

from tenyks.module_loader import make_module_from_file


CLIENT_ROOT = abspath(dirname(__file__))


class NotConfigured(Exception):
    pass


# taken from legit (https://github.com/kennethreitz/legit)
class Settings(object):
    _singleton = {}

    # attributes with defaults
    __attrs__ = tuple()

    def __init__(self, **kwargs):
        super(Settings, self).__init__()

        self.__dict__ = self._singleton

    def __call__(self, *args, **kwargs):
        # new instance of class to call
        r = self.__class__()

        # cache previous settings for __exit__
        r.__cache = self.__dict__.copy()
        map(self.__cache.setdefault, self.__attrs__)

        # set new settings
        self.__dict__.update(*args, **kwargs)

        return r

    def __enter__(self):
        pass

    def __exit__(self, *args):
        # restore cached copy
        self.__dict__.update(self.__cache.copy())
        del self.__cache

    def __getattribute__(self, key):
        if key in object.__getattribute__(self, '__attrs__'):
            try:
                return object.__getattribute__(self, key)
            except AttributeError:
                return None
        return object.__getattribute__(self, key)


settings = Settings()

def collect_settings(settings_path=None, client_name=None):
    errors = []
    intrl_settings = None
    if not len(sys.argv) > 1:
        message = """
You need to provide a settings module.

Use `tcmkconfig clientname > /path/to/settings.py`
        """.format(pr=CLIENT_ROOT)
        raise NotConfigured(message)
    intrl_settings = make_module_from_file('settings', sys.argv[1])

    if client_name and not getattr(settings, 'CLIENT_NAME', None):
        setattr(settings, 'CLIENT_NAME', client_name)

    for sett in filter(lambda x: not x.startswith('__'), dir(intrl_settings)):
        setattr(settings, sett, getattr(intrl_settings, sett))

    if not hasattr(intrl_settings, 'WORKING_DIR'):
        WORKING_DIR = getattr(settings, 'WORKING_DIRECTORY_PATH',
                join(os.environ['HOME'], '.config', settings.CLIENT_NAME))
        DATA_WORKING_DIR = join(WORKING_DIR, 'data')
        try:
            os.makedirs(DATA_WORKING_DIR)
        except:
            errors.append('Could not create {0}'.format(DATA_WORKING_DIR))

        setattr(settings, 'WORKING_DIR', WORKING_DIR)
        setattr(settings, 'DATA_WORKING_DIR', DATA_WORKING_DIR)

    if not hasattr(intrl_settings, 'LOG_DIR'):
        LOG_DIR = WORKING_DIR
        setattr(settings, 'LOG_DIR', LOG_DIR)

    if not hasattr(settings, 'LOG_LEVEL'):
        if settings.DEBUG:
            settings.LOG_LEVEL = 'DEBUG'
        else:
            settings.LOG_LEVEL = 'INFO'

    if hasattr(settings, 'LOG_TO'):
        available_handlers = ('console', 'file', 'syslog')
        if settings.LOG_TO not in available_handlers:
            raise ConfigurationError('LOG_TO must be one of {handlers}'.format(
                handlers=available_handlers))
    else:
        setattr(settings, 'LOG_TO', 'console')

    SYSLOG_PATH = None
    if os.uname()[0] == 'Linux':
        SYSLOG_PATH = '/dev/log'
    elif os.uname()[0] == 'Darwin':
        SYSLOG_PATH = '/var/run/syslog'
    setattr(settings, 'SYSLOG_PATH', SYSLOG_PATH)

    if hasattr(intrl_settings, 'LOGGING_CONFIG'):
        LOGGING_CONFIG = intrl_settings.LOGGING_CONFIG
    else:
        LOGGING_CONFIG = {
            'version': 1,
            'disable_existing_loggers': True,
            'formatters': {
                'color': {
                    'class': 'tenyks.logs.ColorFormatter',
                    'format': '%(asctime)s %(name)s:%(levelname)s %(message)s'
                },
                'default': {
                    'format': '%(asctime)s %(name)s:%(levelname)s %(message)s'
                }
            },
            'handlers': {},
            'loggers': {
                settings.CLIENT_NAME: {
                    'handlers': [settings.LOG_TO],
                    'level': settings.LOG_LEVEL,
                    'propagate': True
                },
            }
        }
        if settings.LOG_TO == 'console':
            LOGGING_CONFIG['handlers']['console'] = {
                'level': settings.LOG_LEVEL,
                'class': 'logging.StreamHandler',
                'formatter': 'color'
            }
        elif settings.LOG_TO == 'syslog':
            LOGGING_CONFIG['handlers']['syslog'] = {
                'address': settings.SYSLOG_PATH,
                'level': settings.LOG_LEVEL,
                'class': 'logging.handlers.SysLogHandler',
                'formatter': 'default',
                'facility': SysLogHandler.LOG_SYSLOG,
            }
        elif settings.LOG_TO == 'file':
            LOGGING_CONFIG['handlers']['file'] = {
                'level': settings.LOG_LEVEL,
                'class': 'logging.FileHandler',
                'formatter': 'default',
                'filename': join(settings.LOG_DIR, 'tenyks.log')
            }


    setattr(settings, 'LOGGING_CONFIG', LOGGING_CONFIG)

    logging.config.dictConfig(LOGGING_CONFIG)

    return errors


def make_config():
    usage = 'tcmkconfig clientname'
    if len(sys.argv) < 2:
        print(usage)
        exit(1)
    with open(join(CLIENT_ROOT, 'settings.py.dist'), 'r') as f:
        settings_template = f.read()
    template = Template(settings_template)
    print(template.render({'client_name': sys.argv[1]}))



from datetime import datetime
import re

import logging
logger = logging.getLogger('tenyks')

from tenyks.utils import parse_irc_prefix
from tenyks import commands

def irc_parse(robot, connection, data):
    command_re = r'^(:(?P<prefix>\S+) )?(?P<cmd>\S+)( (?!:)(?P<args>.+?))?( :(?P<trail>.+))?$'
    match_obj = re.match(command_re, data)
    data = {
        'prefix': match_obj.group('prefix'),
        'command': match_obj.group('cmd'),
        'args': match_obj.group('args'),
        'trail': match_obj.group('trail'),
        'raw': data
    }
    return data

def irc_extract(robot, connection, data):
    if data['command'] == 'PRIVMSG':
        data.update(parse_irc_prefix(data['prefix']))
        message = data['trail']
        data['mask'] = '{user}@{host}'.format(user=data['user'],
            host=data['host'])
        data['connection'] = connection.name
        data['target'] = data['args']
        data['from_channel'] = data['target'].startswith('#')
        data['full_message'] = message
        data['direct'] = (
                message.startswith(connection.nick) or
                not data['from_channel'])

        # TODO CLEAN THIS SHIT UP
        if data['direct'] and data['from_channel']:
            data['private_message'] = False
            data['payload'] = ' '.join(message.split()[1:])
        elif data['from_channel']:
            data['private_message'] = False
            data['payload'] = data['full_message']
        else:
            data['private_message'] = True
            data['payload'] = data['full_message']
    return data

def irc_autoreply(robot, connection, data):
    if data['command'] == 'PING':
        connection.last_ping = datetime.now()
        logger.debug(
            '{connection} autoreply Middleware: last_ping: {dt}'.format(
                connection=connection.name, dt=connection.last_ping))
        reply = data['raw'].replace('PING', 'PONG')
        connection.output_queue.put(reply)
    # Authenticated to server
    elif data['command'] == '001':
        if connection.config.get('commands'):
            for command in connection.config['commands']:
                robot.run_command(connection, command)
        for channel in connection.config['channels']:
            password = ''
            if ',' in channel:
                channel, password = channel.split(',')
            connection.send(commands.JOIN(
                channel=channel.strip(),
                password=password.strip()))
    # Nickname in use
    elif data['command'] == '433':
        nicks = connection.config.get('nicks')
        offset = (nicks.index(connection.nick) + 1) % len(nicks)
        connection.nick = connection.config['nicks'][offset]
        connection.send(commands.NICK(nick=connection.nick))
    return data

def admin_middlware(robot, connection, data):
    conf = connection.config
    data['admin'] = data.get('nick') in conf['admins']
    return data

CORE_MIDDLEWARE = (
    irc_parse,
    irc_extract,
    irc_autoreply,
    admin_middlware
)



from datetime import datetime
import os
import time

import logging

import gevent
from gevent import socket
from gevent import queue
from gevent import ssl


def get_certs_bundle():
    return os.path.join(os.path.dirname(__file__), 'cacert.pem')


class Connection(object):

    def __init__(self, name, **config):
        self.name = name
        self.config = config
        self.connect_attempts = 0
        self.using_ssl = ('ssl' in config and config['ssl'])
        self.greenlets = []
        self.socket = self._fetch_socket()
        self.socket_connected = False
        self.server_disconnect = False
        self.user_disconnect = False
        self.input_queue = queue.Queue()
        self.input_buffer = ''
        self.output_queue = queue.Queue()
        self.output_buffer = ''
        self.logger = logging.getLogger('tenyks.connection.' + self.name)

    def _fetch_socket(self):
        if self.using_ssl:
            if 'ssl_version' in self.config and self.config['ssl_version']:
                if self.config['ssl_version'] == 2:
                    ssl_version = ssl.PROTOCOL_SSLv2
                elif self.config['ssl_version'] == 3:
                    ssl_version = ssl.PROTOCOL_SSLv3
                else:
                    ssl_version = ssl.PROTOCOL_SSLv23
            else:
                # SSLv23 is the default for python ssl and gevent ssl
                ssl_version = ssl.PROTOCOL_SSLv23
            return ssl.wrap_socket(
                socket.socket(),
                cert_reqs=ssl.CERT_REQUIRED,
                ca_certs=get_certs_bundle(),
                ssl_version=ssl_version)
        else:
            return socket.socket()

    def connect(self, reconnecting=False):
        while True:
            self.connect_attempts += 1
            max_retries = self.config.get('retries')
            if max_retries and (self.connect_attempts > max_retries):
                raise Exception("retries exceeded")
            try:
                if reconnecting:
                    self.logger.info('Reconnecting...')
                else:
                    self.logger.info('Connecting...')
                self.socket.connect((self.config['host'],
                    self.config['port']))
                self.socket_connected = True
                self.server_disconnect = False
                self.last_ping = datetime.now()
                self.logger.info('Successfully connected')
                break
            except socket.error as e:
                self.logger.warning('Could not connect: retrying...')
                self.logger.debug(unicode(e, errors='replace'))
                time.sleep(5)

    def post_connect(self):
        self.connect_attempts = 0
        self.spawn_send_and_recv_loops()

    def reconnect(self):
        for greenlet in self.greenlets:
            greenlet.kill()
        self.socket.close()
        self.socket = self._fetch_socket()
        self.connect(reconnecting=True)

    @property
    def needs_reconnect(self):
        return not self.socket_connected and self.server_disconnect

    def close(self):
        self.socket.close()
        self.user_disconnect = True

    def spawn_send_and_recv_loops(self):
        self.greenlets.append(gevent.spawn(self.recv_loop))
        self.greenlets.append(gevent.spawn(self.send_loop))

    def recv_loop(self):
        while True:
            data = self.socket.recv(1024).decode('utf-8')
            if not data:
                self.logger.info('disconnected')
                self.socket_connected = False
                self.server_disconnect = True
                break
            self.logger.debug(u'<- IRC: {data}'.format(data=data))
            self.input_buffer += data
            while '\r\n' in self.input_buffer:
                line, self.input_buffer = self.input_buffer.split('\r\n', 1)
                self.input_queue.put(line)

    def send(self, message):
        """
        send a message to an IRC connection
        """
        message = str(message).strip()
        self.output_queue.put(message)
        self.logger.info(u'Connection -> {connection}: {message}'.format(
            connection=self, message=message))

    def send_loop(self):
        while True:
            line = self.output_queue.get()
            self.output_buffer += line.encode('utf-8', 'replace') + '\r\n'
            while self.output_buffer:
                sent = self.socket.send(self.output_buffer)
                self.logger.info(u'-> IRC: {data}'.format(
                    data=self.output_buffer))
                self.output_buffer = self.output_buffer[sent:]
                time.sleep(.5)



import gevent.monkey
gevent.monkey.patch_all()

import json
import re

import gevent
import redis

import logging

from .config import settings, collect_settings


class Client(object):

    irc_message_filters = {}
    name = None
    direct_only = False

    def __init__(self, name):
        self.channels = [settings.BROADCAST_TO_CLIENTS_CHANNEL]
        self.name = name.lower().replace(' ', '')
        if self.irc_message_filters:
            self.re_irc_message_filters = {}
            for name, regexes in self.irc_message_filters.iteritems():
                if not name in self.re_irc_message_filters:
                    self.re_irc_message_filters[name] = []
                if isinstance(regexes, basestring):
                    regexes = [regexes]
                for regex in regexes:
                    if isinstance(regex, str) or isinstance(regex, unicode):
                        self.re_irc_message_filters[name].append(
                            re.compile(regex).match)
                    else:
                        self.re_irc_message_filters[name].append(regex)
        if hasattr(self, 'recurring'):
            gevent.spawn(self.run_recurring)
        self.logger = logging.getLogger(self.name)

    def run_recurring(self):
        self.recurring()
        recurring_delay = getattr(self, 'recurring_delay', 30)
        gevent.spawn_later(recurring_delay, self.run_recurring)

    def run(self):
        r = redis.Redis(**settings.REDIS_CONNECTION)
        pubsub = r.pubsub()
        pubsub.subscribe(self.channels)
        for raw_redis_message in pubsub.listen():
            try:
                if raw_redis_message['data'] != 1L:
                    data = json.loads(raw_redis_message['data'])
                    if self.direct_only and not data.get('direct', None):
                        continue
                    if self.irc_message_filters and 'payload' in data:
                        name, match = self.search_for_match(data['payload'])
                        ignore = (hasattr(self, 'pass_on_non_match')
                                  and self.pass_on_non_match)
                        if match or ignore:
                            self.delegate_to_handle_method(data, match, name)
                    else:
                        gevent.spawn(self.handle, data, None, None)
            except ValueError:
                self.logger.info('Invalid JSON. Ignoring message.')

    def search_for_match(self, message):
        for name, regexes in self.re_irc_message_filters.iteritems():
            for regex in regexes:
                match = regex(message)
                if match:
                    return name, match
        return None, None

    def delegate_to_handle_method(self, data, match, name):
        if hasattr(self, 'handle_{name}'.format(name=name)):
            callee = getattr(self, 'handle_{name}'.format(name=name))
            gevent.spawn(callee, data, match)
        else:
            gevent.spawn(self.handle, data, match, name)

    def handle(self, data, match, filter_name):
        raise NotImplementedError('`handle` needs to be implemented on all '
                                  'Client subclasses.')

    def send(self, message, data=None):
        r = redis.Redis(**settings.REDIS_CONNECTION)
        broadcast_channel = settings.BROADCAST_TO_ROBOT_CHANNEL
        if data:
            to_publish = json.dumps({
                'command': data['command'],
                'client': self.name,
                'payload': message,
                'target': data['target'],
                'connection': data['connection']
            })
        r.publish(broadcast_channel, to_publish)


def run_client(client_class):
    errors = collect_settings()
    client_instance = client_class(settings.CLIENT_NAME)
    for error in errors:
        client_instance.logger.error(error)
    try:
        client_instance.run()
    except KeyboardInterrupt:
        logger = logging.getLogger(client_instance.name)
        logger.info('exiting')




class Command(object):
    template = ''

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def __str__(self):
        return self.template.format(*self.args, **self.kwargs)

class JOIN(Command):
    template = 'JOIN {channel} {password}'

class NICK(Command):
    template = 'NICK {nick}'

class PASS(Command):
    template = 'PASS {password}'

class PRIVMSG(Command):
    template = 'PRIVMSG {target} :{message}'

class QUIT(Command):
    template = 'QUIT :{message}'

class USER(Command):
    template = 'USER {ident} {host} bla :{realname}'





""" Core of tenyks. Contains Robot and IRC/Redis Lines"""
import gevent.monkey
gevent.monkey.patch_all()

from datetime import datetime
import json
import os
import sys

import logging
logger = logging.getLogger('tenyks')

import gevent
from gevent import queue
import redis

from tenyks.banner import startup_banner
from tenyks.commandmapping import command_parser
from tenyks.config import settings, collect_settings
from tenyks.connection import Connection
from tenyks.utils import pubsub_factory
from tenyks.middleware import CORE_MIDDLEWARE
from tenyks import commands


if hasattr(settings, 'MIDDLEWARE'):
    CORE_MIDDLEWARE += settings.MIDDLEWARE


class Robot(object):
    """
    Design Rules:

    * The Robot has access to services to handle commands.
    * Services send and receive data.
    * The Robot's brain is, effectively, the services.
    * The Robot will forever ONLY relay data to and from platforms (IRC, Email)
      to and from services and handle connection to platforms.
    * The Robot will never handle commands sent from platforms or services.
    * _Almost_ everything is evented.
    """

    def __init__(self):
        self.broadcast_queue = queue.Queue()
        gevent.spawn(self.broadcast_loop)
        gevent.spawn(self.handle_incoming_redis_messages)

        self.prepare_environment()

        self.connections = {}
        self.bootstrap_connections()

    def prepare_environment(self):
        # TODO FIX THIS MESS
        try:
            os.mkdir(settings.WORKING_DIR)
        except OSError:
            # Already exists
            pass
        try:
            os.mkdir(settings.DATA_WORKING_DIR)
        except OSError:
            # Already exists
            pass

    def bootstrap_connections(self):
        for name, connection in settings.CONNECTIONS.iteritems():
            conn = Connection(name, **connection)
            self.connections[name] = conn
            conn.connect()
            success = self.wait_for_success(conn)
            if success:
                self.handshake(conn)
            else:
                logger.error(u'{conn} failed to connect or we did not get a response'.format(
                    conn=conn.name))
                continue

    def wait_for_success(self, connection):
        """
        This will look for something that represents a successful connection
        to the server. If it doesn't see one in 5 seconds, it times out and
        the connection is considered unsuccessful.
        """
        with gevent.Timeout(5, False):
            data = connection.socket.recv(1024).decode('utf-8')
            if data:
                return True
        return False

    def handshake(self, connection):
        if connection.config.get('password'):
            connection.send(commands.PASS(
                password=connection.config['password']))
        connection.nick = connection.config['nicks'][0]
        connection.send(commands.NICK(nick=connection.nick))
        connection.send(commands.USER(
            ident=connection.config['ident'],
            host=connection.config['host'],
            realname=connection.config['realname']))
        connection.post_connect()

    def run_command(self, connection, command):
        connection.send(command_parser(command))

    def say(self, connection, message, channels=[]):
        for channel in channels:
            self.connections[connection].send(commands.PRIVMSG(
                target=channel,
                message=message))

    def broadcast_loop(self):
        """
        Pop things off the broadcast_queue and create jobs for them.
        """
        while True:
            data = self.broadcast_queue.get()
            gevent.spawn(self.broadcast_worker, data)

    def broadcast_worker(self, data):
        """
        This worker will broadcast a message to the service broadcast channel
        """
        r = redis.Redis(**settings.REDIS_CONNECTION)
        broadcast_channel = getattr(settings, 'BROADCAST_TO_SERVICES_CHANNEL',
            'tenyks.services.broadcast_to')
        r.publish(broadcast_channel, json.dumps(data))

    def handle_incoming_redis_messages(self):
        """
        {
            'payload': 'this is a message to IRC',
            'target': '#test',
            'command': 'PRIVMSG',
            'connection': 'freenode',
        }
        """
        broadcast_channel = getattr(settings, 'BROADCAST_TO_ROBOT_CHANNEL',
            'tenyks.robot.broadcast_to')
        pubsub = pubsub_factory(broadcast_channel)
        for raw_redis_message in pubsub.listen():
            logger.info(u'Robot <- {data}'.format(
                data=json.dumps(raw_redis_message)))
            try:
                if raw_redis_message['data'] != 1L:
                    message = json.loads(raw_redis_message['data'])
                    if message['command'].lower() == 'privmsg':
                        payload = message['payload'].replace('\n', '')
                        payload = payload.replace('\r', '')
                        if message.get('private_message', False):
                            target = message['nick']
                        else:
                            target = message['target']
                        self.say(message['connection'],
                                 payload,
                                 channels=[target,])
            except ValueError:
                logger.info('Robot Pubsub: invalid JSON. Ignoring message.')

    def middleware_message(self, connection, data):
        for middleware in CORE_MIDDLEWARE:
            data = middleware(self, connection, data)
        return data

    def connection_worker(self, connection):
        while True:
            if connection.user_disconnect:
                break
            needs_reconnect = connection.needs_reconnect
            ping_delta = datetime.now() - connection.last_ping
            no_ping = ping_delta.seconds > 5 * 60
            if needs_reconnect or no_ping:
                connection.reconnect()
                success = self.wait_for_success(connection)
                if success:
                    self.handshake(connection)
                else:
                    logger.error(u'{conn} failed to connect or we did not get a response'.format(
                        conn=connection.name))
                    continue
            try:
                raw_line = connection.input_queue.get(timeout=5)
            except queue.Empty:
                continue
            data = self.middleware_message(connection, raw_line)
            if data:
                self.broadcast_queue.put(data)
        logger.info(u'{connection} Connection Worker: worker shutdown'.format(
            connection=connection.name))

    def run(self):
        try:
            while True:
                self.workers = []
                for name, connection in self.connections.iteritems():
                    self.workers.append(
                            gevent.spawn(self.connection_worker, connection))
                gevent.joinall(self.workers)
                break
        except KeyboardInterrupt:
            logger.info('Robot: shutting down: user disconnect')
            for name, connection in self.connections.iteritems():
                connection.close()
        finally:
            for name, connection in self.connections.iteritems():
                connection.send(commands.QUIT(
                            message=getattr(self, 'exit_message', 'I\'m out!')))
                connection.close()
            sys.exit('Bye.')


def main():
    collect_settings()
    print(startup_banner())
    robot = Robot()
    robot.run()


if __name__ == '__main__':
    main()



import logging

from termcolors import colorize


COLORS = {
    'DEBUG': ('white', ''),
    'INFO': ('green', ''),
    'WARNING': ('yellow', ''),
    'CRITICAL': ('red', ''),
    'ERROR': ('white', 'red'),
}


class ColorFormatter(logging.Formatter):

    def format(self, record):
        if record.levelname in COLORS:
            colors = COLORS[record.levelname]
            kwargs = {'fg': colors[0]}
            if colors[1]:
                kwargs['bg'] = colors[1]
            record.msg = colorize(record.msg, **kwargs)
        if record.levelname == 'INFO':
            if 'success' in record.msg or 'OK' in record.msg:
                record.msg = colorize(record.msg, fg='green')
        return super(ColorFormatter, self).format(record)



import re

from redis import Redis

from tenyks.config import settings

def pubsub_factory(channel):
    """
    Returns a Redis.pubsub object subscribed to `channel`.
    """
    rdb = Redis(**settings.REDIS_CONNECTION)
    pubsub = rdb.pubsub()
    pubsub.subscribe(channel)
    return pubsub

irc_prefix_re = re.compile(r':?([^!@]*)!?([^@]*)@?(.*)').match
def parse_irc_prefix(prefix):
     nick, user, host = irc_prefix_re(prefix).groups()
     return {
        'nick': nick, 
        'user': user, 
        'host': host
    }
